@@ -1,3 +1,28 @@
+"""Plotters and formatoption of the psy-maps psyplot plugin."""
+
+# Disclaimer
+# ----------
+#
+# Copyright (C) 2021 Helmholtz-Zentrum Hereon
+# Copyright (C) 2020-2021 Helmholtz-Zentrum Geesthacht
+# Copyright (C) 2016-2021 University of Lausanne
+#
+# This file is part of psy-maps and is released under the GNU LGPL-3.O license.
+# See COPYING and COPYING.LESSER in the root of the repository for full
+# licensing details.
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License version 3.0 as
+# published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU LGPL-3.0 license for more details.
+#
+# You should have received a copy of the GNU LGPL-3.0 license
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
+
 import six
 import re
 import warnings
@@ -31,6 +56,9 @@ wrap_proj_types = (ccrs._RectangularProjection,
                    ccrs.Mercator)
 
 
+cartopy_version = list(map(int, cartopy.__version__.split(".")[:2]))
+
+
 @docstrings.get_sections(base='shiftdata')
 def shiftdata(lonsin, datain, lon_0):
     """
@@ -702,7 +730,8 @@ class LonLatBox(BoxBase):
             data = data[0]
         decoder = data.psy.decoder
         lon, lat = self._get_lola(data, decoder)
-        new_lonlatbox = self.calc_lonlatbox(lon, lat)
+        new_lonlatbox = self.calc_lonlatbox(
+            lon, lat, decoder.is_unstructured(data))
         update = self.data_lonlatbox != new_lonlatbox
         if not update and set_data and self.value is not None:
             self.update(self.value)
@@ -744,7 +773,8 @@ class LonLatBox(BoxBase):
     def update_array(self, value, data, decoder, base_var=None):
         """Update the given `data` array"""
         lon, lat = self._get_lola(data, decoder)
-        self.data_lonlatbox = self.calc_lonlatbox(lon, lat)
+        self.data_lonlatbox = self.calc_lonlatbox(
+            lon, lat, decoder.is_unstructured(data))
         if value is None:
             self.lonlatbox = self.data_lonlatbox
             return data
@@ -793,7 +823,8 @@ class LonLatBox(BoxBase):
                     ret = self.mask_outside(data.copy(True), lon, lat, *value,
                                             is_unstructured=is_unstructured)
             lon, lat = self._get_lola(ret, decoder)
-            self.data_lonlatbox = self.calc_lonlatbox(lon, lat)
+            self.data_lonlatbox = self.calc_lonlatbox(
+                lon, lat, is_unstructured)
             return ret
 
     def to_degree(self, units=None, *args):
@@ -844,7 +875,7 @@ class LonLatBox(BoxBase):
                 data.values[mask] = np.nan
         return data
 
-    def calc_lonlatbox(self, lon, lat):
+    def calc_lonlatbox(self, lon, lat, is_unstructured=False):
         if isinstance(self.transform.projection, ccrs.PlateCarree):
             with warnings.catch_warnings():
                 warnings.filterwarnings('ignore', 'invalid value encountered',
@@ -853,7 +884,7 @@ class LonLatBox(BoxBase):
                 lat = lat[np.all([lat >= -90, lat <= 90], axis=0)]
                 return [lon.min(), lon.max(), lat.min(), lat.max()]
         else:
-            if lon.ndim == 1:
+            if lon.ndim == 1 and not is_unstructured:
                 lon, lat = np.meshgrid(lon, lat)
             points = ccrs.PlateCarree().transform_points(
                 self.transform.projection, lon, lat)
@@ -1418,7 +1449,10 @@ class GridLabelSize(Formatoption):
             gl.xlabel_style['size'] = value
             gl.ylabel_style['size'] = value
             try:
-                texts = [t[-1] for t in gl.label_artists]
+                if cartopy_version < [0, 19]:
+                    texts = [t[-1] for t in gl.label_artists]
+                else:
+                    texts = gl.label_artists
             except AttributeError:  # cartopy < 0.18
                 texts = chain(gl.xlabel_artists, gl.ylabel_artists)
             for text in texts:
@@ -1522,8 +1556,13 @@ class GridBase(psyps.DataTicksCalculator):
             return
         gl = self._gridliner
         try:
-            artists = chain(gl.xline_artists, gl.yline_artists,
-                            [t[-1] for t in gl.label_artists])
+            if cartopy_version < [0, 19]:
+                artists = chain(gl.xline_artists, gl.yline_artists,
+                                [t[-1] for t in gl.label_artists])
+            else:
+                artists = chain(
+                    gl.xline_artists, gl.yline_artists, gl.label_artists
+                )
         except AttributeError:  # cartopy < 0.17
             artists = chain(gl.xline_artists, gl.yline_artists,
                             gl.xlabel_artists, gl.ylabel_artists)
@@ -1725,7 +1764,8 @@ class MapPlot2D(psyps.Plot2D):
                     yb = yb[~mask]
             self.logger.debug('Making plot with %i cells', arr.size)
             transformed = proj.transform_points(
-                t, xb.ravel(), yb.ravel())[..., :2].reshape(xb.shape + (2,))
+                t, xb.ravel(), yb.ravel())[...,:2].reshape(xb.shape + (2,))
+            arr = arr.ravel()
             self._plot = PolyCollection(
                 transformed, array=arr.ravel(),
                 norm=self.bounds.norm, rasterized=True, cmap=cmap,
@@ -1734,8 +1774,6 @@ class MapPlot2D(psyps.Plot2D):
             self.ax.add_collection(self._plot)
             if cartopy.__version__ <= "0.18":
                 self._plot.set_clip_path(self.ax.outline_patch)
-            else:
-                self._plot.set_clip_path(self.ax.spines["geo"])
             if wrapped_arr is not None:
                 self.logger.debug('Making wrapped plot with %i cells',
                                   wrapped_arr.size)
@@ -1748,8 +1786,6 @@ class MapPlot2D(psyps.Plot2D):
                 self.ax.add_collection(self._wrapped_plot)
                 if cartopy.__version__ <= "0.18":
                     self._wrapped_plot.set_clip_path(self.ax.outline_patch)
-                else:
-                    self._wrapped_plot.set_clip_path(self.ax.spines["geo"])
         self.logger.debug('Done.')
 
     def remove(self, *args, **kwargs):
